library(fda)
library(MASS)
library(Matrix)

### This Function is modified by Ruzong Fan, March 20, 2013 ###
mflm_beta_smooth_only=function(pheno, mode = "Additive", geno, pos, order, basis, covariate, base = "bspline", interaction = FALSE)  
   {
   geno[is.na(geno)]=0
   covariate[is.na(covariate)] = 0 
   ### define genotyping matrix for Dom and Rec modes ###
   ### For Dom mode, redefine geno[i,j] = 1 if geno[i,j] = 1 or 2
   ### For Rec mode, redefine geno[i,j] = 1 if geno[i,j] = 2 
   geno_X = geno * 0
   for (i in 1:nrow(geno))
      for (j in 1:ncol(geno))
         {
         if (mode == "Dom")
            {
            if (geno[i, j] == 1 || geno[i, j] == 2)
               geno_X[i,j] = 1
            }
            else if ( mode == "Rec")
               if (geno[i, j] == 2)
                  geno_X[i, j] = 1
         }
         
   if  ( mode == "Rec" || mode == "Dom")
      geno = geno_X     
      
   #idx     = is.na(pheno)
   #pheno   = pheno[!idx]
   #geno    = geno[!idx,]
   #covariate = covariate[!idx,]
   
   dqr     = qr(geno)
   index   = dqr$pivot[1:dqr$rank]
   geno    = geno[, index]
   pos     = pos[index]
   nsample = nrow(geno)
   nsnp    = ncol(geno)

   if(max(pos) > 1) {
	  	pos = (pos - min(pos)) / (max(pos) - min(pos))
	    }
      
   if (base ==  "bspline"){
      betabasis  = create.bspline.basis(norder = order, nbasis = basis)
      } else if (base == "fspline"){
      betabasis  = create.fourier.basis(c(0,1), nbasis = basis)
      }else { }
 	    
   B = eval.basis(pos, betabasis)
	
   UJ = geno %*% B
   #########
   pval = list()
     	
   if (interaction == FALSE) 
       {        
       fit        = lm (pheno ~ covariate + UJ)
       if ( dim(pheno)[2] > 1){
          pval$Pillai           = anova(fit, test = "Pillai")[3,6]
          pval$Wilks            = anova(fit, test = "Wilks")[3,6]
          pval$Hotelling_Lawley = anova(fit, test = "Hotelling-Lawley")[3,6]
          pval$Roy              = anova(fit, test = "Roy")[3,6]
          pval$Spherical        = anova(fit, test = "Spherical")[3,5]
          pval$Spherical_GG     = anova(fit, test = "Spherical")[3,6]
          pval$Spherical_HF     = anova(fit, test = "Spherical")[3,7]
          }
          else if ( dim(pheno)[2] == 1){ ### In this case, the following results are the same ###
             pval$F                = anova(fit, test = "F")[2,5]
             pval$Pillai           = anova(fit, test = "Pillai")[2,5]
             pval$Wilks            = anova(fit, test = "Wilks")[2,5]
             pval$Hotelling_Lawley = anova(fit, test = "Hotelling-Lawley")[2,5]
             pval$Roy              = anova(fit, test = "Roy")[2,5]
             pval$Spherical        = anova(fit, test = "Spherical")[2,5]
             } 
       }else ### The following codes are for dim(pheno)[2] > 1 only ####
          {
          Interaction = matrix (0, ncol =  ncol(UJ), nrow =nrow(UJ) )
          for ( i in 1:nrow(Interaction) ){ Interaction[i,] = covariate[i,2] * UJ[i,] }
          
          fitInt     = lm (pheno ~ covariate + UJ + Interaction)
          pval$Pillai           = anova(fitInt, test = "Pillai")[4,6]
          pval$Wilks            = anova(fitInt, test = "Wilks")[4,6]
          pval$Hotelling_Lawley = anova(fitInt, test = "Hotelling-Lawley")[4,6]
          pval$Roy              = anova(fitInt, test = "Roy")[4,6]
          pval$Spherical        = anova(fitInt, test = "Spherical")[4,5]                   
          pval$Spherical_GG     = anova(fitInt, test = "Spherical")[4,6]
          pval$Spherical_HF     = anova(fitInt, test = "Spherical")[4,7]             
          } 

   pval     	
   }

